/*
 *         Files.c
 *
 *         Inherit-file for file-manipulating.
 *         Author: Brainiac (pimentel@fwi.uva.nl)
 *         Last mod: 4-9-91
 *
 *         Calls included in this package:
 *            - seek_line(line, file)
 *            - is_dir(path)
 *            - file_exist(path)
 *            - append_buffer(buffer, file, no_app)
 *            - file_info(file)
 *
 *         You are encouraged by the author to use this inherit-package.
 *         Copying/mutating it, however, without the author's explicit
 *         permission is prohibited.
 *                       
 *         (C) by Brainiac (Andy Pimentel), 1991.
 */

#include "lib.h"

inherit "/lib/string";

seek_line(line, file)
/*
 * Get one particulary line from a file
 * Syntax: string seek(int, string)
 * Returns the line from the file as a string.
 */
{
   string buff;

   if ((line && !intp(line)) || !stringp(file) || line < 1)
   {
      write("Bad argument(s) to seek_line()\n");
      return 1;
   }
   buff = read_file(file);
   if (!buff)
   {
      write("File not found or protected\n");
      return 1;
   }
   if (line > sizeof(buff))
   {
      write("Not that many lines in file!\n");
      return 1;
   }
   return buff[line - 1];
}

is_dir(path)
/*
 * Determines whether a path is a directory
 * Syntax: status is_dir(string)
 * Returns true when path was directory, else returns false.
 */
{
   if (!stringp(path))
   {
      write("Bad argument to is_dir()\n");
      return 0;
   }
   return file_size(path) == -2;
}

file_exist(path)
/*
 * Determines whether a file exists or not 
 * Syntax: status file_exist(string)
 * Returns true when file exists, else returns false.
 */
{  
   if (!stringp(path))
   {
      write("Bad argument to file_exist()\n");
      return 0;
   }
   return file_size(path) != -1;
}

append_buffer(buffer, file, no_app)
/*
 * Appends a buffer (array) to a file
 * Syntax: append_buffer(array, string, status [optional])
 * When setting the "no_app"-flag (no-append), writing will be aborted when
 * the file already exists, thus only new files are created.
 */
{
   int i;

   if (!stringp(file) || !buffer || !pointerp(buffer)
       || (no_app && !intp(no_app)))
   {
      write("Bad argument(s) to append_buffer()\n");
      return 1;
   }
   if (file_exist(file) && no_app)
   {
      write("File already exists...aborted.\n");
      return 1;
   }
   for (i = 0; i < sizeof(buffer); i++)
   {
      write_file(file, strip_newline(buffer[i]) + "\n");
   }
}

file_info(file)
/*
 * Get file-info: file-size and number of lines in file
 * Syntax: array file_info(string)
 * Returns an array of 2 elements: index0 gives file_size, index1 # of lines,
 * if the file exists/is readable. Else it returns 0.
 * Note: when # of lines is 500, this number may be higher because the max-
 * size of arrays in read_file() is 500.
 */
{
   int info;
   string buffer;

   if (!stringp(file))
   {
      write("Bad argument to file_info()\n");
      return 1;
   }
   buffer = read_file(file);
   if (!buffer)
       return 0;  /* no status */
   info = allocate(2);
   info[0] = file_size(file);
   info[1] = sizeof(buffer);

   return info;
}
   
