/*
 *         List.c
 *
 *         Inherit-file for list-manipulating.
 *         Author: Brainiac (pimentel@fwi.uva.nl)
 *         Last mod: 4-9-91
 *
 *         Calls included in this package:
 *           - add_to_list(elt, list)
 *           - insert_at(elt, index, list)
 *           - del_from_list(elt, list)
 *           - del_index(index, list)
 *           - inlist(elt, list)
 *           - type_match(var1, var2)
 *           - filter(elt, list)
 *           - swap_elts(index1, index2, list)
 *           - car(list)
 *           - cdr(list)
 *           - write_list(list)
 *
 *         You are encouraged by the author to use this inherit-package.
 *         Copying/mutating it, however, without the author's explicit
 *         permission is prohibited.
 *                       
 *         (C) by Brainiac (Andy Pimentel), 1991.
 */

#include "lib.h"

inherit "/lib/string";

add_to_list(elt, list)
/*
 * Add an element to a list (array).
 * Syntax: array add_to_list(element, array)
 * Returns the new list.
 */
{
   if (list && !pointerp(list))
   {
      write("Bad argument to add_to_list()\n");
      return 1;
   }
   if (list && sizeof(list) == MAX_ELTS)
   {
      write("List is full in add_to_list()\n");
      return 1;
   }
   return (!list) ? ({elt}) : list + ({elt});
}

insert_at(elt, i, list)
/*
 * Inserts an element at position i in a list. The old element at position 
 * i shifts up to the i+1's position
 * Syntax: array insert_at(elt, int, array)
 * Returns the new list.
 */
{
   if (!list || !pointerp(list) || (i && !intp(i))) 
   {
      write("Bad argument(s) to insert_at()\n");
      return 1;
   }
   if (i < 0 || i >= sizeof(list))
   {
      write("Index does not exist in insert_at()\n");
      return 1;
   }
   if (sizeof(list) == MAX_ELTS)
   {
      write("List is full in insert_at()\n");
      return 1;
   }
   return (i == 0) ? ({elt}) + list : 
		     slice_array(list, 0, i - 1) + ({elt}) + 
	             slice_array(list, i, sizeof(list) - 1);
}

del_index(i, list)
/*
 * Deletes element on index i in a list
 * Syntax: array del_index(int, array)
 * Returns the new list.
 */
{
   if (!list || !pointerp(list) || (i && !intp(i))) 
   {
      write("Bad argument(s) to del_index()\n");
      return 1;
   }
   if (i == 0)
       return (sizeof(list) > 0) ? slice_array(list, 1, sizeof(list) - 1) : 0;
   else if (i == sizeof(list) - 1)
       return slice_array(list, 0, i - 1);      
   else
       return slice_array(list, 0, i - 1) + 
	      slice_array(list, i + 1, sizeof(list) - 1);
}

del_from_list(elt, list)
/*
 * Delete an element from a list (array).
 * Syntax: array del_from_list(element, array)
 * Returns the new list.
 */
{
   int i;

   if (!list || !pointerp(list))
   {
      write("Bad argument to del_from_list()\n");
      return 1;
   }
   if (sizeof(list))
      do { i++; } while (i < sizeof(list) && list[i] != elt);
   if (i == sizeof(list))
   {
      write("Element in del_from_list() not found!\n");
      return list;
   }
   return del_index(i, list);
}

inlist(elt, list)
/* 
 * Checks whether elt is in the list
 * Syntax: int inlist(elt, array)
 * Returns elt's position when present, else it returns -1.  
 */
{
   int i;

   if (!list)
      return -1;
   if (!pointerp(list))
   {
      write("Bad argument to inlist()\n");
      return -1; 
   }

   while(1)
   {
      if (list[i] == elt && type_match(list[i], elt))
	 return i;
      if (i < sizeof(list) - 1) i++;
      else return -1;
   }
}

type_match(var1, var2)
/*
 * Checks whether types of var1 and var2 are equivalent
 * Syntax: status type_match(variable1, variable2)
 * Returns true when equivalent, else false.
 */
{
   if (
        (intp(var1) && intp(var1)) ||
	(stringp(var1) && stringp(var2)) ||
	(objectp(var1) && objectp(var2)) ||
	(pointerp(var1) && pointerp(var2)) 
      ) return 1;

   return 0;
}

swap_elts(i, j, list)
/*
 * Swap the elements on indices i and j in a list
 * Syntax: array swap_elts(int, int, array)
 * Returns the new list.
 * Note: because LPC already offers a swap() call, my call is renamed.
 */
{
   string temp; /* Yeah, this *is* possible in LPC, because 2.* has no
		   strict type-checking. It's a dirty trick though :-) */

   if ( !list || !pointerp(list) ||
	(i && !intp(i)) ||
	(j && !intp(j)) )
   {
      write("Bad argument(s) to swap_elts()\n");
      return 1;
   }
   temp = list[j];
   list[j] = list[i];
   list[i] = temp;

   return list;
}

car(list)
/*
 * Get the first element of a list
 * Syntax: elt car(array) (Scheme notation)
 * Returns the first element.
 */
{
   if (!list)
   {
      write("Empty list in car()\n");
      return 1;
   }
   if (!pointerp(list))
   {
      write("Bad argument to car()\n");
      return 1;
   }
   return list[0];
}


cdr(list)
/*
 * Get the tail-elements (elts after first elt) of a list
 * Syntax: array cdr(array) (Scheme notation)
 * Returns the tail.
 */
{
   if (!list)
   {
      write("Empty list in cdr()\n");
      return 1;
   }
   if (!pointerp(list))
   {
      write("Bad argument to cdr()\n");
      return 1;
   }
   return (sizeof(list) > 0) ? slice_array(list, 1, sizeof(list) - 1) : 0;
}


filter(elt, list)
/*
 * Filters 'elt' out of a list
 * Syntax: array filter(elt, array)
 * Returns the array without any element 'elt'.
 */
{
   int position;

   if (!list || !pointerp(list))
   {
      write("Bad argument to filter()\n");
      return 1;
   }

   while ((position = inlist(elt, list)) != -1)
      list = del_index(position, list);
   return list;
}

write_list(list)
/*
 * Writes a list on the screen
 * Syntax: write_list(array)
 * Note: short list call! (see inlist())
 */
{
   int i;

   if (!list || !pointerp(list))
   {
      write("Bad argument to write_list()\n");
      return 1;
   }
   for (i = 0; i < sizeof(list); i++)
       if (stringp(list[i]))
          write(strip_newline(list[i]) + "\n");
       else
	  { write(list[i]); write("\n"); } 
}
